@@ -26,7 +26,7 @@
 #include <config.h>
 #include "core.h"
 #include "frame-private.h"
-#include "workspace.h"
+#include "workspace-private.h"
 #include "prefs.h"
 
 /* Looks up the MetaWindow representing the frame of the given X window.
@@ -145,6 +145,13 @@ meta_core_get (Display *xdisplay,
             case META_WINDOW_DOCK:
             case META_WINDOW_TOOLBAR:
             case META_WINDOW_SPLASHSCREEN:
+	    case META_WINDOW_DROPDOWN_MENU:
+	    case META_WINDOW_POPUP_MENU:
+	    case META_WINDOW_TOOLTIP:
+	    case META_WINDOW_NOTIFICATION:
+	    case META_WINDOW_COMBO:
+	    case META_WINDOW_DND:
+	    case META_WINDOW_OVERRIDE_OTHER:
               /* No frame */
               base_type = META_FRAME_TYPE_LAST;
               break;
