@@ -37,6 +37,8 @@
 #include "common.h"
 #include "boxes.h"
 #include "display.h"
+#include "keybindings-private.h"
+#include "prefs.h"
 
 #ifdef HAVE_STARTUP_NOTIFICATION
 #include <libsn/sn.h>
@@ -46,10 +48,8 @@
 #include <X11/extensions/sync.h>
 #endif
 
-typedef struct _MetaKeyBinding MetaKeyBinding;
 typedef struct _MetaStack      MetaStack;
 typedef struct _MetaUISlave    MetaUISlave;
-typedef struct _MetaWorkspace  MetaWorkspace;
 
 typedef struct _MetaGroupPropHooks  MetaGroupPropHooks;
 
@@ -74,6 +74,8 @@ typedef void (* MetaWindowPingFunc) (MetaDisplay *display,
 
 struct _MetaDisplay
 {
+  GObject parent_instance;
+  
   char *name;
   Display *xdisplay;
 
@@ -213,6 +215,8 @@ struct _MetaDisplay
   unsigned int hyper_mask;
   unsigned int super_mask;
   unsigned int meta_mask;
+  MetaKeyCombo overlay_key_combo;
+  gboolean overlay_key_only_pressed;
   
   /* Xinerama cache */
   unsigned int xinerama_cache_invalidated : 1;
@@ -297,6 +301,11 @@ struct _MetaDisplay
 #endif
 };
 
+struct _MetaDisplayClass
+{
+  GObjectClass parent_class;
+};
+
 /* Xserver time can wraparound, thus comparing two timestamps needs to take
  * this into account.  Here's a little macro to help out.  If no wraparound
  * has occurred, this is equivalent to
@@ -370,20 +379,6 @@ void     meta_display_set_grab_op_cursor (MetaDisplay *display,
                                           Window       grab_xwindow,
                                           guint32      timestamp);
 
-gboolean meta_display_begin_grab_op (MetaDisplay *display,
-                                     MetaScreen  *screen,
-                                     MetaWindow  *window,
-                                     MetaGrabOp   op,
-                                     gboolean     pointer_already_grabbed,
-                                     gboolean     frame_action,
-                                     int          button,
-                                     gulong       modmask,
-                                     guint32      timestamp,
-                                     int          root_x,
-                                     int          root_y);
-void     meta_display_end_grab_op   (MetaDisplay *display,
-                                     guint32      timestamp);
-
 void    meta_display_check_threshold_reached (MetaDisplay *display,
                                               int          x,
                                               int          y);
@@ -406,9 +401,6 @@ void     meta_display_increment_event_serial (MetaDisplay *display);
 
 void     meta_display_update_active_window_hint (MetaDisplay *display);
 
-guint32  meta_display_get_current_time           (MetaDisplay *display);
-guint32  meta_display_get_current_time_roundtrip (MetaDisplay *display);
-
 /* utility goo */
 const char* meta_event_mode_to_string   (int m);
 const char* meta_event_detail_to_string (int d);
@@ -428,36 +420,6 @@ void meta_display_ping_window      (MetaDisplay        *display,
 gboolean meta_display_window_has_pending_pings (MetaDisplay        *display,
 						MetaWindow         *window);
 
-typedef enum
-{
-  META_TAB_LIST_NORMAL,
-  META_TAB_LIST_DOCKS,
-  META_TAB_LIST_GROUP
-} MetaTabList;
-
-typedef enum
-{
-  META_TAB_SHOW_ICON,      /* Alt-Tab mode */
-  META_TAB_SHOW_INSTANTLY  /* Alt-Esc mode */
-} MetaTabShowType;
-
-GList* meta_display_get_tab_list (MetaDisplay   *display,
-                                  MetaTabList    type,
-                                  MetaScreen    *screen,
-                                  MetaWorkspace *workspace);
-
-MetaWindow* meta_display_get_tab_next (MetaDisplay   *display,
-                                       MetaTabList    type,
-				       MetaScreen    *screen,
-                                       MetaWorkspace *workspace,
-                                       MetaWindow    *window,
-                                       gboolean       backward);
-
-MetaWindow* meta_display_get_tab_current (MetaDisplay   *display,
-                                          MetaTabList    type,
-                                          MetaScreen    *screen,
-                                          MetaWorkspace *workspace);
-
 int meta_resize_gravity_from_grab_op (MetaGrabOp op);
 
 gboolean meta_grab_op_is_moving   (MetaGrabOp op);
@@ -471,30 +433,10 @@ void meta_display_increment_focus_sentinel (MetaDisplay *display);
 void meta_display_decrement_focus_sentinel (MetaDisplay *display);
 gboolean meta_display_focus_sentinel_clear (MetaDisplay *display);
 
-/* meta_display_set_input_focus_window is like XSetInputFocus, except
- * that (a) it can't detect timestamps later than the current time,
- * since Metacity isn't part of the XServer, and thus gives erroneous
- * behavior in this circumstance (so don't do it), (b) it uses
- * display->last_focus_time since we don't have access to the true
- * Xserver one, (c) it makes use of display->user_time since checking
- * whether a window should be allowed to be focused should depend
- * on user_time events (see bug 167358, comment 15 in particular)
- */
-void meta_display_set_input_focus_window   (MetaDisplay *display, 
-                                            MetaWindow  *window,
-                                            gboolean     focus_frame,
-                                            guint32      timestamp);
-
-/* meta_display_focus_the_no_focus_window is called when the
- * designated no_focus_window should be focused, but is otherwise the
- * same as meta_display_set_input_focus_window
- */
-void meta_display_focus_the_no_focus_window (MetaDisplay *display, 
-                                             MetaScreen  *screen,
-                                             guint32      timestamp);
-
 void meta_display_queue_autoraise_callback  (MetaDisplay *display,
                                              MetaWindow  *window);
 void meta_display_remove_autoraise_callback (MetaDisplay *display);
 
+void meta_display_overlay_key_activate (MetaDisplay *display);
+
 #endif
