@@ -225,22 +225,22 @@ read_rgb_icon (MetaDisplay   *display,
   gulong nitems;
   gulong bytes_after;
   int result, err;
-  gulong *data;
+  guchar *data;
   gulong *best;
   int w, h;
   gulong *best_mini;
   int mini_w, mini_h;
+  gulong *data_as_long;
 
   meta_error_trap_push_with_return (display);
   type = None;
   data = NULL;
   result = XGetWindowProperty (display->xdisplay,
 			       xwindow,
-                               display->atom_net_wm_icon,
+                               display->atom__NET_WM_ICON,
 			       0, G_MAXLONG,
 			       False, XA_CARDINAL, &type, &format, &nitems,
-			       &bytes_after, ((guchar **)&data));
-
+			       &bytes_after, &data);
   err = meta_error_trap_pop_with_return (display, TRUE);
 
   if (err != Success ||
@@ -253,7 +253,9 @@ read_rgb_icon (MetaDisplay   *display,
       return FALSE;
     }
 
-  if (!find_best_size (data, nitems,
+  data_as_long = (gulong *)data;
+
+  if (!find_best_size (data_as_long, nitems,
                        ideal_width, ideal_height,
                        &w, &h, &best))
     {
@@ -261,7 +263,7 @@ read_rgb_icon (MetaDisplay   *display,
       return FALSE;
     }
 
-  if (!find_best_size (data, nitems,
+  if (!find_best_size (data_as_long, nitems,
                        ideal_mini_width, ideal_mini_height,
                        &mini_w, &mini_h, &best_mini))
     {
@@ -464,6 +466,7 @@ get_kwm_win_icon (MetaDisplay *display,
   int format;
   gulong nitems;
   gulong bytes_after;
+  guchar *data;
   Pixmap *icons;
   int err, result;
 
@@ -473,19 +476,20 @@ get_kwm_win_icon (MetaDisplay *display,
   meta_error_trap_push_with_return (display);
   icons = NULL;
   result = XGetWindowProperty (display->xdisplay, xwindow,
-                               display->atom_kwm_win_icon,
+                               display->atom__KWM_WIN_ICON,
 			       0, G_MAXLONG,
 			       False,
-                               display->atom_kwm_win_icon,
+                               display->atom__KWM_WIN_ICON,
 			       &type, &format, &nitems,
-			       &bytes_after, (guchar **)&icons);
+			       &bytes_after, &data);
+  icons = (Pixmap *)data;
 
   err = meta_error_trap_pop_with_return (display, TRUE);
   if (err != Success ||
       result != Success)
     return;
 
-  if (type != display->atom_kwm_win_icon)
+  if (type != display->atom__KWM_WIN_ICON)
     {
       XFree (icons);
       return;
@@ -556,9 +560,9 @@ meta_icon_cache_property_changed (MetaIconCache *icon_cache,
                                   MetaDisplay   *display,
                                   Atom           atom)
 {
-  if (atom == display->atom_net_wm_icon)
+  if (atom == display->atom__NET_WM_ICON)
     icon_cache->net_wm_icon_dirty = TRUE;
-  else if (atom == display->atom_kwm_win_icon)
+  else if (atom == display->atom__KWM_WIN_ICON)
     icon_cache->kwm_win_icon_dirty = TRUE;
   else if (atom == XA_WM_HINTS)
     icon_cache->wm_hints_dirty = TRUE;
@@ -817,30 +821,14 @@ meta_read_icons (MetaScreen     *screen,
   if (icon_cache->want_fallback &&
       icon_cache->origin < USING_FALLBACK_ICON)
     {
-      GdkPixbuf *fallback_icon;
-      GdkPixbuf *fallback_mini_icon;
-      
-      fallback_icon = NULL;
-      fallback_mini_icon = NULL;
+      get_fallback_icons (screen,
+                          iconp,
+                          ideal_width,
+                          ideal_height,
+                          mini_iconp,
+                          ideal_mini_width,
+                          ideal_mini_height);
 
-      meta_ui_get_fallback_icons(&fallback_icon, &fallback_mini_icon);
-
-      if (fallback_icon == NULL || fallback_mini_icon == NULL)
-        {
-          get_fallback_icons (screen,
-                              iconp,
-                              ideal_width,
-                              ideal_height,
-                              mini_iconp,
-                              ideal_mini_width,
-                              ideal_mini_height);
-        }
-
-      if (fallback_icon != NULL)
-        *iconp = fallback_icon;
-      if (fallback_mini_icon != NULL)
-        *mini_iconp = fallback_mini_icon;
-      
       replace_cache (icon_cache, USING_FALLBACK_ICON,
                      *iconp, *mini_iconp);
       
