@@ -52,8 +52,10 @@
 #include "ui.h"
 #include "session.h"
 #include "prefs.h"
+#include "compositor.h"
 
 #include <glib-object.h>
+#include <gdk/gdkx.h>
 
 #include <stdlib.h>
 #include <sys/types.h>
@@ -66,6 +68,16 @@
 #include <locale.h>
 #include <time.h>
 
+#ifdef WITH_CLUTTER
+#include <clutter/clutter.h>
+#include <clutter/x11/clutter-x11.h>
+#endif
+
+#ifdef HAVE_INTROSPECTION
+#include <girepository.h>
+#include "compositor/mutter/mutter-plugin-manager.h"
+#endif
+
 /**
  * The exit code we'll return to our parent process when we eventually die.
  */
@@ -213,12 +225,15 @@ typedef struct
   gchar *save_file;
   gchar *display_name;
   gchar *client_id;
+  gchar *mutter_plugins;
   gboolean replace_wm;
   gboolean disable_sm;
   gboolean print_version;
   gboolean sync;
   gboolean composite;
   gboolean no_composite;
+  gboolean no_tab_popup;
+  gchar *introspect;
 } MetaArguments;
 
 #ifdef HAVE_COMPOSITE_EXTENSIONS
@@ -240,11 +255,11 @@ typedef struct
  * \param argv  Pointer to the array of arguments Metacity was given
  * \param meta_args  The result of parsing the arguments.
  **/
-static void
+static GOptionContext *
 meta_parse_options (int *argc, char ***argv,
                     MetaArguments *meta_args)
 {
-  MetaArguments my_args = {NULL, NULL, NULL,
+  MetaArguments my_args = {NULL, NULL, NULL, NULL,
                            FALSE, FALSE, FALSE, FALSE, FALSE};
   GOptionEntry options[] = {
     {
@@ -300,6 +315,27 @@ meta_parse_options (int *argc, char ***argv,
       N_("Turn compositing off"),
       NULL
     },
+#ifdef WITH_CLUTTER
+    {
+      "mutter-plugins", 0, 0, G_OPTION_ARG_STRING,
+      &my_args.mutter_plugins,
+      N_("Comma-separated list of compositor plugins"),
+      "PLUGINS"
+    },
+#endif
+    {
+      "no-tab-popup", 0, 0, G_OPTION_ARG_NONE,
+      &my_args.no_tab_popup,
+      N_("Whether window popup/frame should be shown when cycling windows."),
+      NULL
+    },
+#ifdef HAVE_INTROSPECTION
+    {
+      "introspect-dump", 0, 0, G_OPTION_ARG_STRING,
+      &my_args.introspect,
+      N_("Internal argument for GObject introspection"), "INTROSPECT"
+    },
+#endif
     {NULL}
   };
   GOptionContext *ctx;
@@ -307,16 +343,95 @@ meta_parse_options (int *argc, char ***argv,
 
   ctx = g_option_context_new (NULL);
   g_option_context_add_main_entries (ctx, options, "metacity");
+  
+#ifdef WITH_CLUTTER
+  /*
+   * This function is only available in clutter >= 0.8.2
+   */
+#if CLUTTER_CHECK_VERSION(0,8,2)
+  g_option_context_add_group (ctx, clutter_get_option_group_without_init ());
+#endif
+#endif
+
   if (!g_option_context_parse (ctx, argc, argv, &error))
     {
       g_print ("metacity: %s\n", error->message);
       exit(1);
     }
-  g_option_context_free (ctx);
+
   /* Return the parsed options through the meta_args param. */
   *meta_args = my_args;
+  return ctx;
 }
 
+
+#ifdef WITH_CLUTTER
+/* Metacity is responsible for pulling events off the X queue, so Clutter
+ * doesn't need (and shouldn't) run its normal event source which polls
+ * the X fd, but we do have to deal with dispatching events that accumulate
+ * in the clutter queue. This happens, for example, when clutter generate
+ * enter/leave events on mouse motion - several events are queued in the
+ * clutter queue but only one dispatched. It could also happen because of
+ * explicit calls to clutter_event_put(). We add a very simple custom
+ * event loop source which is simply responsible for pulling events off
+ * of the queue and dispatching them before we block for new events.
+ */
+
+static gboolean 
+event_prepare (GSource    *source,
+               gint       *timeout_)
+{
+  *timeout_ = -1;
+
+  return clutter_events_pending ();
+}
+
+static gboolean 
+event_check (GSource *source)
+{
+  return clutter_events_pending ();
+}
+
+static gboolean
+event_dispatch (GSource    *source,
+                GSourceFunc callback,
+                gpointer    user_data)
+{
+  ClutterEvent *event = clutter_event_get ();
+  if (event)
+    clutter_do_event (event);
+
+  return TRUE;
+}
+
+static GSourceFuncs event_funcs = {
+  event_prepare,
+  event_check,
+  event_dispatch
+};
+
+static void
+meta_clutter_init (GOptionContext *ctx, int *argc, char ***argv)
+{
+  clutter_x11_set_display (gdk_display);
+  clutter_x11_disable_event_retrieval ();
+  
+  if (CLUTTER_INIT_SUCCESS == clutter_init (argc, argv))
+    {
+      meta_compositor_can_use_clutter__ = 1;
+
+      GSource *source = g_source_new (&event_funcs, sizeof (GSource));
+      g_source_attach (source, NULL);
+      g_source_unref (source);
+    }
+  else
+    {
+      g_message ("Unable to initialize Clutter.\n");
+      meta_compositor_can_use_clutter__ = 0;
+    }
+}
+#endif
+
 /**
  * Selects which display Metacity should use. It first tries to use
  * display_name as the display. If display_name is NULL then
@@ -375,7 +490,8 @@ main (int argc, char **argv)
     "Pango", "GLib-GObject", "GThread"
   };
   guint i;
-
+  GOptionContext *ctx;
+  
   if (!g_thread_supported ())
     g_thread_init (NULL);
   
@@ -410,6 +526,8 @@ main (int argc, char **argv)
       meta_warning ("Could not change to home directory %s.\n",
                     g_get_home_dir ());
 
+  g_type_init ();
+
   meta_print_self_identity ();
   
   bindtextdomain (GETTEXT_PACKAGE, METACITY_LOCALEDIR);
@@ -417,7 +535,57 @@ main (int argc, char **argv)
   textdomain (GETTEXT_PACKAGE);
 
   /* Parse command line arguments.*/
-  meta_parse_options (&argc, &argv, &meta_args);
+  ctx = meta_parse_options (&argc, &argv, &meta_args);
+
+#ifdef WITH_CLUTTER
+  /* This must come before the introspect below, so we load all the plugins
+   * in order to get their get_type functions.
+   */
+  if (meta_args.mutter_plugins)
+    {
+      char **plugins = g_strsplit (meta_args.mutter_plugins, ",", -1);
+      char **plugin;
+      GSList *plugins_list = NULL;
+
+      for (plugin = plugins; *plugin; plugin++)
+        {
+          g_strstrip (*plugin);
+          plugins_list = g_slist_prepend (plugins_list, *plugin);
+        }
+
+      plugins_list = g_slist_reverse (plugins_list);
+      meta_prefs_override_clutter_plugins (plugins_list);
+
+      g_slist_free(plugins_list);
+      g_strfreev (plugins);
+    }
+#endif /* WITH_CLUTTER */
+
+#ifdef HAVE_INTROSPECTION
+  g_irepository_prepend_search_path (METACITY_PKGLIBDIR);  
+  if (meta_args.introspect)
+    {
+      GError *error = NULL;
+      if (meta_args.mutter_plugins)
+        {
+          /* We need to load all plugins so that we can call their
+           * get_type functions.  We do not call
+           * mutter_plugin_manager_initialize because almost nothing else
+           * is initialized at this point, and we don't plan to run any real
+           * plugin code.
+           */
+          MutterPluginManager *mgr = mutter_plugin_manager_new (NULL);
+          if (!mutter_plugin_manager_load (mgr))
+            g_critical ("failed to load plugins");
+        }
+      if (!g_irepository_dump (meta_args.introspect, &error))
+        {
+          g_printerr ("failed to dump: %s\n", error->message);
+          return 1;
+        }
+      return 0;
+    }
+#endif
 
   meta_set_syncing (meta_args.sync || (g_getenv ("METACITY_SYNC") != NULL));
 
@@ -434,10 +602,17 @@ main (int argc, char **argv)
   
   meta_main_loop = g_main_loop_new (NULL, FALSE);
   
-  g_type_init ();
-
   meta_ui_init (&argc, &argv);  
 
+#ifdef WITH_CLUTTER
+  /*
+   * Clutter can only be initialized after the UI.
+   */
+  meta_clutter_init (ctx, &argc, &argv);
+#endif
+
+  g_option_context_free (ctx);
+
   /* must be after UI init so we can override GDK handlers */
   meta_errors_init ();
 
@@ -526,6 +701,11 @@ main (int argc, char **argv)
   if (meta_args.composite || meta_args.no_composite)
     meta_prefs_set_compositing_manager (meta_args.composite);
 
+  if (meta_args.no_tab_popup)
+    {
+      meta_prefs_override_no_tab_popup (TRUE);
+    }
+
   if (!meta_display_open ())
     meta_exit (META_EXIT_ERROR);
   
