@@ -33,11 +33,12 @@
 #include "window-private.h"
 #include "frame-private.h"
 #include "prefs.h"
-#include "workspace.h"
-#include "keybindings.h"
+#include "workspace-private.h"
+#include "keybindings-private.h"
 #include "stack.h"
 #include "xprops.h"
 #include "compositor.h"
+#include "alttabhandlerdefault.h"
 
 #ifdef HAVE_SOLARIS_XINERAMA
 #include <X11/extensions/xinerama.h>
@@ -69,6 +70,124 @@ static void meta_screen_sn_event   (SnMonitorEvent *event,
                                     void           *user_data);
 #endif
 
+enum
+{
+  PROP_N_WORKSPACES = 1,
+  PROP_KEYBOARD_GRABBED,
+};
+
+enum
+{
+  RESTACKED,
+  TOGGLE_RECORDING,
+
+  LAST_SIGNAL
+};
+
+static guint screen_signals[LAST_SIGNAL] = { 0 };
+
+G_DEFINE_TYPE (MetaScreen, meta_screen, G_TYPE_OBJECT);
+
+static void
+meta_screen_set_property (GObject      *object,
+                          guint         prop_id,
+                          const GValue *value,
+                          GParamSpec   *pspec)
+{
+#if 0
+  MetaScreen *screen = META_SCREEN (object);
+#endif
+
+  switch (prop_id)
+    {
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+static void
+meta_screen_get_property (GObject      *object,
+                          guint         prop_id,
+                          GValue       *value,
+                          GParamSpec   *pspec)
+{
+  MetaScreen *screen = META_SCREEN (object);
+
+  switch (prop_id)
+    {
+    case PROP_N_WORKSPACES:
+      g_value_set_int (value, meta_screen_get_n_workspaces (screen));
+      break;
+    case PROP_KEYBOARD_GRABBED:
+      g_value_set_boolean (value, screen->all_keys_grabbed ? TRUE : FALSE);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+static void
+meta_screen_finalize (GObject *object)
+{
+  /* Actual freeing done in meta_screen_free() for now */
+}
+
+static void
+meta_screen_class_init (MetaScreenClass *klass)
+{
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  GParamSpec   *pspec;
+
+  object_class->get_property = meta_screen_get_property;
+  object_class->set_property = meta_screen_set_property;
+  object_class->finalize = meta_screen_finalize;
+
+  screen_signals[RESTACKED] =
+    g_signal_new ("restacked",
+                  G_TYPE_FROM_CLASS (object_class),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (MetaScreenClass, restacked),
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
+
+  pspec = g_param_spec_int ("n-workspaces",
+                            "N Workspaces",
+                            "Number of workspaces",
+                            1, G_MAXINT, 1,
+                            G_PARAM_READABLE);
+
+  screen_signals[TOGGLE_RECORDING] =
+    g_signal_new ("toggle-recording",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  0,
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
+
+  g_object_class_install_property (object_class,
+                                   PROP_N_WORKSPACES,
+                                   pspec);
+
+  pspec = g_param_spec_boolean ("keyboard-grabbed",
+                                "Keyboard grabbed",
+                                "Whether the keyboard is grabbed",
+                                FALSE,
+                                G_PARAM_READABLE);
+
+  g_object_class_install_property (object_class,
+                                   PROP_KEYBOARD_GRABBED,
+                                   pspec);
+}
+
+static void
+meta_screen_init (MetaScreen *screen)
+{
+}
+
 static int
 set_wm_check_hint (MetaScreen *screen)
 {
@@ -311,6 +430,38 @@ reload_xinerama_infos (MetaScreen *screen)
   g_assert (screen->xinerama_infos != NULL);
 }
 
+/* The guard window allows us to leave minimized windows mapped so
+ * that compositor code may provide live previews of them.
+ * Instead of being unmapped/withdrawn, they get pushed underneath
+ * the guard window. */
+static Window
+create_guard_window (Display *xdisplay, MetaScreen *screen)
+{
+  XSetWindowAttributes attributes;
+  Window guard_window;
+  
+  attributes.event_mask = NoEventMask;
+  attributes.override_redirect = True;
+  attributes.background_pixel = BlackPixel (xdisplay, screen->number);
+
+  guard_window =
+    XCreateWindow (xdisplay,
+		   screen->xroot,
+		   0, /* x */
+		   0, /* y */
+		   screen->rect.width,
+		   screen->rect.height,
+		   0, /* border width */
+		   CopyFromParent, /* depth */
+		   CopyFromParent, /* class */
+		   CopyFromParent, /* visual */
+		   CWEventMask|CWOverrideRedirect|CWBackPixel,
+		   &attributes);
+  XLowerWindow (xdisplay, guard_window);
+  XMapWindow (xdisplay, guard_window);
+  return guard_window;
+}
+
 MetaScreen*
 meta_screen_new (MetaDisplay *display,
                  int          number,
@@ -456,7 +607,7 @@ meta_screen_new (MetaDisplay *display,
       return NULL;
     }
   
-  screen = g_new (MetaScreen, 1);
+  screen = g_object_new (META_TYPE_SCREEN, NULL);
   screen->closing = 0;
   
   screen->display = display;
@@ -490,6 +641,7 @@ meta_screen_new (MetaDisplay *display,
   screen->vertical_workspaces = FALSE;
   screen->starting_corner = META_SCREEN_TOPLEFT;
   screen->compositor_data = NULL;
+  screen->guard_window = None;
 
   {
     XFontStruct *font_info;
@@ -574,7 +726,8 @@ meta_screen_new (MetaDisplay *display,
   screen->ui = meta_ui_new (screen->display->xdisplay,
                             screen->xscreen);
 
-  screen->tab_popup = NULL;
+  screen->tab_handler = NULL;
+  screen->ws_popup = NULL;
   
   screen->stack = meta_stack_new (screen);
 
@@ -684,7 +837,8 @@ meta_screen_free (MetaScreen *screen,
     g_free (screen->xinerama_infos);
   
   g_free (screen->screen_name);
-  g_free (screen);
+
+  g_object_unref (screen);
 
   XFlush (display->xdisplay);
   meta_display_ungrab (display);
@@ -745,7 +899,11 @@ meta_screen_manage_all_windows (MetaScreen *screen)
   GList *list;
 
   meta_display_grab (screen->display);
-  
+
+  if (screen->guard_window == None)
+    screen->guard_window = create_guard_window (screen->display->xdisplay,
+                                                screen);
+
   windows = list_windows (screen);
 
   meta_stack_freeze (screen->stack);
@@ -756,19 +914,6 @@ meta_screen_manage_all_windows (MetaScreen *screen)
 
       window = meta_window_new_with_attrs (screen->display, info->xwindow, TRUE,
                                            &info->attrs);
-      if (info->xwindow == screen->no_focus_window ||
-          info->xwindow == screen->flash_window ||
-#ifdef HAVE_COMPOSITE_EXTENSIONS
-          info->xwindow == screen->wm_cm_selection_window ||
-#endif
-          info->xwindow == screen->wm_sn_selection_window) {
-        meta_verbose ("Not managing our own windows\n");
-        continue;
-      }
-
-      if (screen->display->compositor)
-        meta_compositor_add_window (screen->display->compositor, window,
-                                    info->xwindow, &info->attrs);
     }
   meta_stack_thaw (screen->stack);
 
@@ -783,41 +928,32 @@ meta_screen_composite_all_windows (MetaScreen *screen)
 {
 #ifdef HAVE_COMPOSITE_EXTENSIONS
   MetaDisplay *display;
-  GList *windows, *list;
+  GSList *windows, *tmp;
 
   display = screen->display;
   if (!display->compositor)
     return;
 
-  windows = list_windows (screen);
-
+  windows = meta_display_list_windows (display);
+  for (tmp = windows; tmp != NULL; tmp = tmp->next)
+    meta_compositor_add_window (display->compositor, tmp->data);
+  g_slist_free (windows);
+  
+  /* trigger a stack_sync_to_server: */
   meta_stack_freeze (screen->stack);
-
-  for (list = windows; list != NULL; list = list->next)
-    {
-      WindowInfo *info = list->data;
-
-      if (info->xwindow == screen->no_focus_window ||
-          info->xwindow == screen->flash_window ||
-          info->xwindow == screen->wm_sn_selection_window ||
-          info->xwindow == screen->wm_cm_selection_window) {
-        meta_verbose ("Not managing our own windows\n");
-        continue;
-      }
-
-      meta_compositor_add_window (display->compositor,
-                                  meta_display_lookup_x_window (display,
-                                                                info->xwindow),
-				  info->xwindow, &info->attrs);
-    }
-
   meta_stack_thaw (screen->stack);
-
-  g_list_foreach (windows, (GFunc)g_free, NULL);
-  g_list_free (windows);
 #endif
 }
 
+/**
+ * meta_screen_for_x_screen:
+ * @xscreen: an X screen structure.
+ *
+ * Gets the #MetaScreen corresponding to an X screen structure.
+ *
+ * Return value: (transfer none): the #MetaScreen for the X screen
+ *   %NULL if Metacity is not managing the screen.
+ */
 MetaScreen*
 meta_screen_for_x_screen (Screen *xscreen)
 {
@@ -978,6 +1114,18 @@ meta_screen_get_n_workspaces (MetaScreen *screen)
   return g_list_length (screen->workspaces);
 }
 
+/**
+ * meta_screen_get_workspace_by_index:
+ * @screen: a #MetaScreen
+ * @index: index of one of the screen's workspaces
+ *
+ * Gets the workspace object for one of a screen's workspaces given the workspace
+ * index. It's valid to call this function with an out-of-range index and it
+ * will robustly return %NULL.
+ *
+ * Return value: (transfer none): the workspace object with specified index, or %NULL
+ *   if the index is out of range.
+ */
 MetaWorkspace*
 meta_screen_get_workspace_by_index (MetaScreen  *screen,
                                     int          idx)
@@ -1071,6 +1219,106 @@ set_desktop_viewport_hint (MetaScreen *screen)
   meta_error_trap_pop (screen->display, FALSE);
 }
 
+void
+meta_screen_remove_workspace (MetaScreen *screen, MetaWorkspace *workspace,
+                              guint32 timestamp)
+{
+  GList         *l;
+  MetaWorkspace *neighbour = NULL;
+  GList         *next = NULL;
+
+  l = screen->workspaces;
+  while (l)
+    {
+      MetaWorkspace *w = l->data;
+
+      if (w == workspace)
+        {
+          if (l->next)
+            next = l->next;
+
+          if (l->prev)
+            neighbour = l->prev->data;
+          else if (l->next)
+            neighbour = l->next->data;
+          else
+            {
+              /* Cannot remove the only workspace! */
+              return;
+            }
+
+          break;
+        }
+
+      l = l->next;
+    }
+
+  if (!neighbour)
+    return;
+
+  meta_workspace_relocate_windows (workspace, neighbour);
+
+  if (workspace == screen->active_workspace)
+    meta_workspace_activate (neighbour, timestamp);
+
+  /* This also removes the workspace from the screens list */
+  meta_workspace_remove (workspace);
+
+  set_number_of_spaces_hint (screen, g_list_length (screen->workspaces));
+
+  l = next;
+  while (l)
+    {
+      MetaWorkspace *w = l->data;
+
+      meta_workspace_update_window_hints (w);
+
+      l = l->next;
+    }
+
+  meta_screen_queue_workarea_recalc (screen);
+
+  g_object_notify (G_OBJECT (screen), "n-workspaces");
+}
+
+/**
+ * meta_screen_append_new_workspace:
+ * @screen: a #MetaScreen
+ * @activate: %TRUE if the workspace should be switched to after creation
+ * @timestamp: if switching to a new workspace, timestamp to be used when
+ *   focusing a window on the new workspace. (Doesn't hurt to pass a valid
+ *   timestamp when available even if not switching workspaces.)
+ *
+ * Append a new workspace to the screen and (optionally) switch to that
+ * screen.
+ *
+ * Return value: (transfer none): the newly appended workspace.
+ */
+MetaWorkspace *
+meta_screen_append_new_workspace (MetaScreen *screen, gboolean activate,
+                                  guint32 timestamp)
+{
+  MetaWorkspace *w;
+
+  /* This also adds the workspace to the screen list */
+  w = meta_workspace_new (screen);
+
+  if (!w)
+    return NULL;
+
+  if (activate)
+    meta_workspace_activate (w, timestamp);
+
+  set_number_of_spaces_hint (screen, g_list_length (screen->workspaces));
+
+  meta_screen_queue_workarea_recalc (screen);
+
+  g_object_notify (G_OBJECT (screen), "n-workspaces");
+
+  return w;
+}
+
+
 static void
 update_num_workspaces (MetaScreen *screen,
                        guint32     timestamp)
@@ -1135,7 +1383,7 @@ update_num_workspaces (MetaScreen *screen,
       MetaWorkspace *w = tmp->data;
 
       g_assert (w->windows == NULL);
-      meta_workspace_free (w);
+      meta_workspace_remove (w);
       
       tmp = tmp->next;
     }
@@ -1151,6 +1399,8 @@ update_num_workspaces (MetaScreen *screen,
   set_number_of_spaces_hint (screen, new_num);
 
   meta_screen_queue_workarea_recalc (screen);
+
+  g_object_notify (G_OBJECT (screen), "n-workspaces");
 }
 
 static void
@@ -1188,188 +1438,68 @@ meta_screen_update_cursor (MetaScreen *screen)
   XFreeCursor (screen->display->xdisplay, xcursor);
 }
 
-#define MAX_PREVIEW_SIZE 150.0
-
-static GdkPixbuf *
-get_window_pixbuf (MetaWindow *window,
-                   int        *width,
-                   int        *height)
-{
-  Pixmap pmap;
-  GdkPixbuf *pixbuf, *scaled;
-  double ratio;
-
-  pmap = meta_compositor_get_window_pixmap (window->display->compositor,
-                                            window);
-  if (pmap == None)
-    return NULL;
-
-  pixbuf = meta_ui_get_pixbuf_from_pixmap (pmap);
-  if (pixbuf == NULL) 
-    return NULL;
-
-  *width = gdk_pixbuf_get_width (pixbuf);
-  *height = gdk_pixbuf_get_height (pixbuf);
-
-  /* Scale pixbuf to max dimension MAX_PREVIEW_SIZE */
-  if (*width > *height)
-    {
-      ratio = ((double) *width) / MAX_PREVIEW_SIZE;
-      *width = (int) MAX_PREVIEW_SIZE;
-      *height = (int) (((double) *height) / ratio);
-    }
-  else
-    {
-      ratio = ((double) *height) / MAX_PREVIEW_SIZE;
-      *height = (int) MAX_PREVIEW_SIZE;
-      *width = (int) (((double) *width) / ratio);
-    }
-
-  scaled = gdk_pixbuf_scale_simple (pixbuf, *width, *height,
-                                    GDK_INTERP_BILINEAR);
-  g_object_unref (pixbuf);
-  return scaled;
-}
-                                         
 void
-meta_screen_ensure_tab_popup (MetaScreen      *screen,
+meta_screen_tab_popup_create (MetaScreen      *screen,
                               MetaTabList      list_type,
-                              MetaTabShowType  show_type)
+                              MetaTabShowType  show_type,
+                              MetaWindow      *initial_selection)
 {
-  MetaTabEntry *entries;
   GList *tab_list;
   GList *tmp;
-  int len;
-  int i;
 
-  if (screen->tab_popup)
-    return;
+  g_return_if_fail (screen->tab_handler == NULL);
+
+  screen->tab_handler = meta_alt_tab_handler_new (screen,
+                                                  show_type == META_TAB_SHOW_INSTANTLY);
 
   tab_list = meta_display_get_tab_list (screen->display,
                                         list_type,
                                         screen,
                                         screen->active_workspace);
   
-  len = g_list_length (tab_list);
-
-  entries = g_new (MetaTabEntry, len + 1);
-  entries[len].key = NULL;
-  entries[len].title = NULL;
-  entries[len].icon = NULL;
+  for (tmp = tab_list; tmp; tmp = tmp->next)
+    meta_alt_tab_handler_add_window (screen->tab_handler, tmp->data);
   
-  i = 0;
-  tmp = tab_list;
-  while (i < len)
-    {
-      MetaWindow *window;
-      MetaRectangle r;
-      GdkPixbuf *win_pixbuf;
-      int width, height;
-
-      window = tmp->data;
-      
-      entries[i].key = (MetaTabEntryKey) window->xwindow;
-      entries[i].title = window->title;
-
-      win_pixbuf = get_window_pixbuf (window, &width, &height);
-      if (win_pixbuf == NULL)
-        entries[i].icon = g_object_ref (window->icon);
-      else
-        {
-          int icon_width, icon_height, t_width, t_height;
-#define ICON_OFFSET 6
-
-          icon_width = gdk_pixbuf_get_width (window->icon);
-          icon_height = gdk_pixbuf_get_height (window->icon);
-
-          t_width = width + ICON_OFFSET;
-          t_height = height + ICON_OFFSET;
-
-          entries[i].icon = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
-                                            t_width, t_height);
-          gdk_pixbuf_fill (entries[i].icon, 0x00000000);
-          gdk_pixbuf_copy_area (win_pixbuf, 0, 0, width, height,
-                                entries[i].icon, 0, 0);
-          g_object_unref (win_pixbuf);
-          gdk_pixbuf_composite (window->icon, entries[i].icon, 
-                                t_width - icon_width, t_height - icon_height,
-                                icon_width, icon_height,
-                                t_width - icon_width, t_height - icon_height, 
-                                1.0, 1.0, GDK_INTERP_BILINEAR, 255);
-        }
-                                
-      entries[i].blank = FALSE;
-      entries[i].hidden = !meta_window_showing_on_its_workspace (window);
-      entries[i].demands_attention = window->wm_state_demands_attention;
-      
-      if (show_type == META_TAB_SHOW_INSTANTLY ||
-          !entries[i].hidden                   ||
-          !meta_window_get_icon_geometry (window, &r))
-        meta_window_get_outer_rect (window, &r);
-      
-      entries[i].rect = r;
-
-      /* Find inside of highlight rectangle to be used when window is
-       * outlined for tabbing.  This should be the size of the
-       * east/west frame, and the size of the south frame, on those
-       * sides.  On the top it should be the size of the south frame
-       * edge.
-       */
-#define OUTLINE_WIDTH 5
-      /* Top side */
-      if (!entries[i].hidden &&
-          window->frame && window->frame->bottom_height > 0 &&
-          window->frame->child_y >= window->frame->bottom_height)
-        entries[i].inner_rect.y = window->frame->bottom_height;
-      else
-        entries[i].inner_rect.y = OUTLINE_WIDTH;
-
-      /* Bottom side */
-      if (!entries[i].hidden &&
-          window->frame && window->frame->bottom_height != 0)
-        entries[i].inner_rect.height = r.height
-          - entries[i].inner_rect.y - window->frame->bottom_height;
-      else
-        entries[i].inner_rect.height = r.height
-          - entries[i].inner_rect.y - OUTLINE_WIDTH;
-
-      /* Left side */
-      if (!entries[i].hidden && window->frame && window->frame->child_x != 0)
-        entries[i].inner_rect.x = window->frame->child_x;
-      else
-        entries[i].inner_rect.x = OUTLINE_WIDTH;
-
-      /* Right side */
-      if (!entries[i].hidden &&
-          window->frame && window->frame->right_width != 0)
-        entries[i].inner_rect.width = r.width
-          - entries[i].inner_rect.x - window->frame->right_width;
-      else
-        entries[i].inner_rect.width = r.width
-          - entries[i].inner_rect.x - OUTLINE_WIDTH;
-      
-      ++i;
-      tmp = tmp->next;
-    }
-
-  screen->tab_popup = meta_ui_tab_popup_new (entries, 
-                                             screen->number,
-                                             len,
-                                             5, /* FIXME */
-                                             TRUE);
-
-  for (i = 0; i < len; i++) 
-    g_object_unref (entries[i].icon);
-
-  g_free (entries);
-
-  g_list_free (tab_list);
-  
-  /* don't show tab popup, since proper window isn't selected yet */
+  meta_alt_tab_handler_show (screen->tab_handler, initial_selection);
 }
 
 void
-meta_screen_ensure_workspace_popup (MetaScreen *screen)
+meta_screen_tab_popup_forward (MetaScreen *screen)
+{
+  g_return_if_fail (screen->tab_handler != NULL);
+
+  meta_alt_tab_handler_forward (screen->tab_handler);
+}
+
+void
+meta_screen_tab_popup_backward (MetaScreen *screen)
+{
+  g_return_if_fail (screen->tab_handler != NULL);
+
+  meta_alt_tab_handler_backward (screen->tab_handler);
+}
+
+MetaWindow *
+meta_screen_tab_popup_get_selected (MetaScreen *screen)
+{
+  g_return_val_if_fail (screen->tab_handler != NULL, NULL);
+
+  return meta_alt_tab_handler_get_selected (screen->tab_handler);
+}
+
+void
+meta_screen_tab_popup_destroy (MetaScreen *screen)
+{
+  g_return_if_fail (screen->tab_handler != NULL);
+
+  meta_alt_tab_handler_destroy (screen->tab_handler);
+  g_object_unref (screen->tab_handler);
+  screen->tab_handler = NULL;
+}
+
+void
+meta_screen_workspace_popup_create (MetaScreen    *screen,
+                                    MetaWorkspace *initial_selection)
 {
   MetaTabEntry *entries;
   int len;
@@ -1378,8 +1508,7 @@ meta_screen_ensure_workspace_popup (MetaScreen *screen)
   int n_workspaces;
   int current_workspace;
   
-  if (screen->tab_popup)
-    return;
+  g_return_if_fail (screen->ws_popup == NULL);
 
   current_workspace = meta_workspace_index (screen->active_workspace);
   n_workspaces = meta_screen_get_n_workspaces (screen);
@@ -1424,16 +1553,46 @@ meta_screen_ensure_workspace_popup (MetaScreen *screen)
       ++i;
     }
 
-  screen->tab_popup = meta_ui_tab_popup_new (entries, 
-                                             screen->number,
-                                             len,
-                                             layout.cols,
-                                             FALSE);      
+  screen->ws_popup = meta_ui_tab_popup_new (entries,
+                                            screen->number,
+                                            len,
+                                            layout.cols,
+                                            FALSE);
 
   g_free (entries);
   meta_screen_free_workspace_layout (&layout);
 
-  /* don't show tab popup, since proper space isn't selected yet */
+  meta_ui_tab_popup_select (screen->ws_popup,
+                            (MetaTabEntryKey) initial_selection);
+  meta_ui_tab_popup_set_showing (screen->ws_popup, TRUE);
+}
+
+void
+meta_screen_workspace_popup_select (MetaScreen    *screen,
+                                    MetaWorkspace *workspace)
+{
+  g_return_if_fail (screen->ws_popup != NULL);
+
+  meta_ui_tab_popup_select (screen->ws_popup,
+                            (MetaTabEntryKey) workspace);
+}
+
+MetaWorkspace *
+meta_screen_workspace_popup_get_selected (MetaScreen *screen)
+{
+  g_return_val_if_fail (screen->ws_popup != NULL, NULL);
+
+  return (MetaWorkspace *) meta_ui_tab_popup_get_selected (screen->ws_popup);
+}
+
+void
+meta_screen_workspace_popup_destroy (MetaScreen *screen)
+{
+  if (screen->ws_popup)
+    {
+      meta_ui_tab_popup_free (screen->ws_popup);
+      screen->ws_popup = NULL;
+    }
 }
 
 MetaWindow*
@@ -2284,6 +2443,10 @@ meta_screen_resize (MetaScreen *screen,
   reload_xinerama_infos (screen);
   set_desktop_geometry_hint (screen);
   
+  if (screen->display->compositor)
+    meta_compositor_sync_screen_size (screen->display->compositor,
+				      screen, width, height);
+
   /* Queue a resize on all the windows */
   meta_screen_foreach_window (screen, meta_screen_resize_func, 0);
 }
@@ -2741,6 +2904,13 @@ meta_screen_get_screen_number (MetaScreen *screen)
   return screen->number;
 }
 
+/**
+ * meta_screen_get_display:
+ * Retrieve the display associated with screen.
+ * @screen: A #MetaScreen
+ * 
+ * Returns: (transfer none): Display 
+ */
 MetaDisplay *
 meta_screen_get_display (MetaScreen *screen)
 {
@@ -2800,3 +2970,32 @@ meta_screen_unset_cm_selection (MetaScreen *screen)
   XSetSelectionOwner (screen->display->xdisplay, a, None, CurrentTime);
 }
 #endif /* HAVE_COMPOSITE_EXTENSIONS */
+
+GList *
+meta_screen_get_workspaces (MetaScreen *screen)
+{
+  return screen->workspaces;
+}
+
+int
+meta_screen_get_active_workspace_index (MetaScreen *screen)
+{
+  MetaWorkspace *active = screen->active_workspace;
+
+  if (!active)
+    return -1;
+
+  return meta_workspace_index (active);
+}
+
+MetaWorkspace *
+meta_screen_get_active_workspace (MetaScreen *screen)
+{
+  return screen->active_workspace;
+}
+
+void
+meta_screen_restacked (MetaScreen *screen)
+{
+  g_signal_emit (screen, screen_signals[RESTACKED], 0);
+}
