@@ -698,6 +698,20 @@ window_type_to_string (MetaWindowType type)
       return "splashscreen";
     case META_WINDOW_UTILITY:
       return "utility";
+    case META_WINDOW_DROPDOWN_MENU:
+      return "dropdown_menu";
+    case META_WINDOW_POPUP_MENU:
+      return "popup_menu";
+    case META_WINDOW_TOOLTIP:
+      return "tooltip";
+    case META_WINDOW_NOTIFICATION:
+      return "notification";
+    case META_WINDOW_COMBO:
+      return "combo";
+    case META_WINDOW_DND:
+      return "dnd";
+    case META_WINDOW_OVERRIDE_OTHER:
+      return "override_redirect";
     }
 
   return "";
@@ -1191,8 +1205,8 @@ start_element_handler  (GMarkupParseContext *context,
               g_set_error (error,
                            G_MARKUP_ERROR,
                            G_MARKUP_ERROR_UNKNOWN_ATTRIBUTE,
-                           _("Unknown attribute %s on <metacity_session> element"),
-                           name);
+                           _("Unknown attribute %s on <%s> element"),
+                           name, "metacity_session");
               return;
             }
           
@@ -1266,8 +1280,8 @@ start_element_handler  (GMarkupParseContext *context,
               g_set_error (error,
                            G_MARKUP_ERROR,
                            G_MARKUP_ERROR_UNKNOWN_ATTRIBUTE,
-                           _("Unknown attribute %s on <window> element"),
-                           name);
+                           _("Unknown attribute %s on <%s> element"),
+                           name, "window");
               session_info_free (pd->info);
               pd->info = NULL;
               return;
@@ -1298,8 +1312,8 @@ start_element_handler  (GMarkupParseContext *context,
               g_set_error (error,
                            G_MARKUP_ERROR,
                            G_MARKUP_ERROR_UNKNOWN_ATTRIBUTE,
-                           _("Unknown attribute %s on <window> element"),
-                           name);
+                           _("Unknown attribute %s on <%s> element"),
+                           name, "window");
               session_info_free (pd->info);
               pd->info = NULL;
               return;
@@ -1370,8 +1384,8 @@ start_element_handler  (GMarkupParseContext *context,
               g_set_error (error,
                            G_MARKUP_ERROR,
                            G_MARKUP_ERROR_UNKNOWN_ATTRIBUTE,
-                           _("Unknown attribute %s on <maximized> element"),
-                           name);
+                           _("Unknown attribute %s on <%s> element"),
+                           name, "maximized");
               return;
             }
 
@@ -1430,8 +1444,8 @@ start_element_handler  (GMarkupParseContext *context,
               g_set_error (error,
                            G_MARKUP_ERROR,
                            G_MARKUP_ERROR_UNKNOWN_ATTRIBUTE,
-                           _("Unknown attribute %s on <geometry> element"),
-                           name);
+                           _("Unknown attribute %s on <%s> element"),
+                           name, "geometry");
               return;
             }
           
