@@ -44,8 +44,7 @@ typedef enum
   META_PREF_TITLEBAR_FONT,
   META_PREF_NUM_WORKSPACES,
   META_PREF_APPLICATION_BASED,
-  META_PREF_WINDOW_KEYBINDINGS,
-  META_PREF_SCREEN_KEYBINDINGS,
+  META_PREF_KEYBINDINGS,
   META_PREF_DISABLE_WORKAROUNDS,
   META_PREF_COMMANDS,
   META_PREF_TERMINAL_COMMAND,
@@ -59,7 +58,14 @@ typedef enum
   META_PREF_GNOME_ANIMATIONS,
   META_PREF_CURSOR_THEME,
   META_PREF_CURSOR_SIZE,
-  META_PREF_COMPOSITING_MANAGER
+  META_PREF_COMPOSITING_MANAGER,
+  META_PREF_RESIZE_WITH_RIGHT_BUTTON,
+#ifdef WITH_CLUTTER
+  META_PREF_CLUTTER_DISABLED,
+  META_PREF_CLUTTER_PLUGINS,
+#endif
+  META_PREF_LIVE_HIDDEN_WINDOWS,
+  META_PREF_NO_TAB_POPUP,
 } MetaPreference;
 
 typedef void (* MetaPrefsChangedFunc) (MetaPreference pref,
@@ -74,6 +80,8 @@ void meta_prefs_init (void);
 const char* meta_preference_to_string (MetaPreference pref);
 
 MetaVirtualModifier         meta_prefs_get_mouse_button_mods  (void);
+guint                       meta_prefs_get_mouse_button_resize (void);
+guint                       meta_prefs_get_mouse_button_menu  (void);
 MetaFocusMode               meta_prefs_get_focus_mode         (void);
 MetaFocusNewWindows         meta_prefs_get_focus_new_windows  (void);
 gboolean                    meta_prefs_get_raise_on_click     (void);
@@ -120,6 +128,37 @@ gboolean    meta_prefs_get_compositing_manager (void);
  */
 void meta_prefs_set_compositing_manager (gboolean whether);
 
+#ifdef WITH_CLUTTER
+
+gboolean meta_prefs_get_clutter_disabled (void);
+void     meta_prefs_set_clutter_disabled (gboolean whether);
+
+GSList * meta_prefs_get_clutter_plugins (void);
+
+/**
+ * Sets the list of plugins persistently in GConf
+ *
+ * \param list   list of plugin:option pairs
+ */
+void meta_prefs_set_clutter_plugins (GSList *list);
+
+/**
+ * Sets the list of plugins temporarily for this process. The value
+ * from GConf will be ignored.
+ *
+ * \param list   list of plugin:option pairs
+ */
+void meta_prefs_override_clutter_plugins (GSList *list);
+
+#endif
+
+gboolean meta_prefs_get_live_hidden_windows (void);
+void     meta_prefs_set_live_hidden_windows (gboolean whether);
+
+gboolean meta_prefs_get_no_tab_popup (void);
+void     meta_prefs_set_no_tab_popup (gboolean whether);
+void     meta_prefs_override_no_tab_popup (gboolean whether);
+
 /* XXX FIXME This should be x-macroed, but isn't yet because it would be
  * difficult (or perhaps impossible) to add the suffixes using the current
  * system.  It needs some more thought, perhaps after the current system
@@ -183,7 +222,9 @@ typedef struct
 typedef struct
 {
   const char   *name;
-  /* a list of MetaKeyCombos. Each of them is bound to
+  const char   *default_keybinding;
+  /**
+   * A list of MetaKeyCombos. Each of them is bound to
    * this keypref. If one has keysym==modifiers==0, it is
    * ignored. For historical reasons, the first entry is
    * governed by the pref FOO and the remainder are
@@ -191,14 +232,15 @@ typedef struct
    */
   GSList *bindings;
 
-  /* for keybindings that can have shift or not like Alt+Tab */
-  gboolean      add_shift;
+  /** for keybindings that can have shift or not like Alt+Tab */
+  gboolean      add_shift:1;
+
+  /** for keybindings that apply only to a window */
+  gboolean      per_window:1;
 } MetaKeyPref;
 
-void meta_prefs_get_screen_bindings (const MetaKeyPref **bindings,
-                                     int                *n_bindings);
-void meta_prefs_get_window_bindings (const MetaKeyPref **bindings,
-                                     int                *n_bindings);
+void meta_prefs_get_key_bindings (const MetaKeyPref **bindings,
+                                  int                *n_bindings);
 
 MetaKeyBindingAction meta_prefs_get_keybinding_action (const char *name);
 
@@ -206,6 +248,8 @@ void meta_prefs_get_window_binding (const char          *name,
                                     unsigned int        *keysym,
                                     MetaVirtualModifier *modifiers);
 
+void meta_prefs_get_overlay_binding (MetaKeyCombo *combo);
+
 typedef enum
 {
   META_VISUAL_BELL_INVALID = 0,
